#pragma once

namespace esphome {
namespace resistance_sampler {

/// Abstract interface to mark components that provide resistance values.
class ResistanceSampler {};

}  // namespace resistance_sampler
}  // namespace esphome
